@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2020 Red Hat, Inc.
+// Copyright (C) 2013-2023 Red Hat, Inc.
 
 /// @file
 
@@ -18,8 +18,8 @@ ABG_END_EXPORT_DECLARATIONS
 namespace abigail
 {
 config::config()
-  : m_format_minor("0"),
-    m_format_major("2"),
+  : m_format_minor(ABIGAIL_ABIXML_VERSION_MINOR),
+    m_format_major(ABIGAIL_ABIXML_VERSION_MAJOR),
     m_xml_element_indent(2),
     m_tu_instr_suffix(".bi"),
     m_tu_instr_archive_suffix(".abi")
@@ -68,6 +68,15 @@ config::set_tu_instr_archive_suffix(const std::string& s)
 
 extern "C"
 {
+/// Return the relevant version numbers of the library.
+///
+/// @param maj the major version number of the library.
+///
+/// @param min the minor version number of the library.
+///
+/// @param rev the revision version number of the library.
+///
+/// @param suf the version suffix of the library.
 void
 abigail_get_library_version(std::string& major,
 			    std::string& minor,
@@ -80,5 +89,18 @@ abigail_get_library_version(std::string& major,
   suffix = ABIGAIL_VERSION_SUFFIX;
 }
 
+/// Return the version numbers for the ABIXML format.
+///
+/// @param maj the major version number of the ABIXML format.
+///
+/// @param min the minor version number of the ABIXML format.
+void
+abigail_get_abixml_version(std::string& major,
+			   std::string& minor)
+{
+  major = ABIGAIL_ABIXML_VERSION_MAJOR;
+  minor = ABIGAIL_ABIXML_VERSION_MINOR;
+}
+
 }
 }//end namespace abigail
