@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2014-2020 Red Hat, Inc.
+// Copyright (C) 2014-2023 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -37,9 +37,13 @@
 #include "abg-config.h"
 #include "abg-tools-utils.h"
 #include "abg-corpus.h"
+#include "abg-reader.h"
 #include "abg-dwarf-reader.h"
 #include "abg-comparison.h"
 #include "abg-suppression.h"
+#ifdef WITH_CTF
+#include "abg-ctf-reader.h"
+#endif
 
 using std::string;
 using std::cerr;
@@ -49,6 +53,8 @@ using std::ofstream;
 using std::vector;
 using std::shared_ptr;
 
+using namespace abigail;
+
 using abigail::tools_utils::emit_prefix;
 
 class options
@@ -71,7 +77,14 @@ public:
   bool			list_undefined_symbols_only;
   bool			show_base_names;
   bool			show_redundant;
+  bool			redundant_opt_set;
+  bool			no_redundant_opt_set;
   bool			show_locs;
+  bool			fail_no_debug_info;
+  bool			ignore_soname;
+#ifdef WITH_CTF
+  bool			use_ctf;
+#endif
 
   options(const char* program_name)
     :prog_name(program_name),
@@ -81,7 +94,15 @@ public:
      list_undefined_symbols_only(),
      show_base_names(),
      show_redundant(true),
-     show_locs(true)
+     redundant_opt_set(),
+     no_redundant_opt_set(),
+     show_locs(true),
+     fail_no_debug_info(),
+     ignore_soname(false)
+#ifdef WITH_CTF
+    ,
+      use_ctf()
+#endif
   {}
 }; // end struct options
 
@@ -105,12 +126,17 @@ display_usage(const string& prog_name, ostream& out)
     "to the debug information directory for the first library\n"
     << "  --lib-debug-info-dir2|--libd2 <path-to-lib-debug-info2>  set the path "
     "to the debug information directory for the second library\n"
-    <<  "--suppressions|--suppr <path> specify a suppression file\n"
-    << "--no-redundant  do not display redundant changes\n"
-    << "--no-show-locs  do now show location information\n"
-    << "--redundant  display redundant changes (this is the default)\n"
-    << "--weak-mode  check compatibility between the application and "
-    "just one version of the library."
+    << "  --suppressions|--suppr <path> specify a suppression file\n"
+    << "  --no-redundant  do not display redundant changes\n"
+    << "  --no-show-locs  do now show location information\n"
+    << "  --ignore-soname  do not take the SONAMEs into account\n"
+    << "  --fail-no-debug-info  bail out if no debug info was found\n"
+    << "  --redundant  display redundant changes (this is the default)\n"
+    << "  --weak-mode  check compatibility between the application and "
+    "just one version of the library.\n"
+#ifdef WITH_CTF
+    << "  --ctf use CTF instead of DWARF in ELF files\n"
+#endif
     ;
 }
 
@@ -191,11 +217,21 @@ parse_command_line(int argc, char* argv[], options& opts)
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--redundant"))
-	opts.show_redundant = true;
+        {
+	  opts.show_redundant = true;
+	  opts.redundant_opt_set = true;
+	}
       else if (!strcmp(argv[i], "--no-redundant"))
-	opts.show_redundant = false;
+        {
+  	  opts.show_redundant = false;
+	  opts.no_redundant_opt_set = true;
+	}
       else if (!strcmp(argv[i], "--no-show-locs"))
 	opts.show_locs = false;
+      else if (!strcmp(argv[i], "--ignore-soname"))
+	opts.ignore_soname=true;
+      else if (!strcmp(argv[i], "--fail-no-debug-info"))
+	opts.fail_no_debug_info = true;
       else if (!strcmp(argv[i], "--help")
 	       || !strcmp(argv[i], "-h"))
 	{
@@ -204,6 +240,10 @@ parse_command_line(int argc, char* argv[], options& opts)
 	}
       else if (!strcmp(argv[i], "--weak-mode"))
 	opts.weak_mode = true;
+#ifdef WITH_CTF
+      else if (!strcmp(argv[i], "--ctf"))
+        opts.use_ctf = true;
+#endif
       else
 	{
 	  opts.unknow_option = argv[i];
@@ -236,8 +276,6 @@ using abigail::ir::type_base_sptr;
 using abigail::ir::function_type_sptr;
 using abigail::ir::function_decl;
 using abigail::ir::var_decl;
-using abigail::dwarf_reader::status;
-using abigail::dwarf_reader::read_corpus_from_elf;
 using abigail::comparison::diff_context_sptr;
 using abigail::comparison::diff_context;
 using abigail::comparison::diff_sptr;
@@ -267,6 +305,8 @@ create_diff_context(const options& opts)
   ctxt->show_linkage_names(true);
   ctxt->show_redundant_changes(opts.show_redundant);
   ctxt->show_locs(opts.show_locs);
+  // Intentional logic flip of ignore_soname
+  ctxt->show_soname_change(!opts.ignore_soname);
   ctxt->switch_categories_off
     (abigail::comparison::ACCESS_CHANGE_CATEGORY
      | abigail::comparison::COMPATIBLE_TYPE_CHANGE_CATEGORY
@@ -390,7 +430,7 @@ perform_compat_check_in_normal_mode(options& opts,
   return status;
 }
 
-/// An description of a change of the type of a function.  It contains
+/// A description of a change of the type of a function.  It contains
 /// the declaration of the function we are interested in, as well as
 /// the differences found in the type of that function.
 struct fn_change
@@ -600,6 +640,74 @@ perform_compat_check_in_weak_mode(options& opts,
   return status;
 }
 
+/// Read an ABI corpus, be it from ELF or abixml.
+///
+/// @param opts the options passed from the user to the program.
+///
+/// @param status the resulting fe_iface::status to send back to the
+/// caller.
+///
+/// @param di_roots the directories from where to look for debug info.
+///
+/// @param env the environment used for libabigail.
+///
+/// @param path the path to the ABI corpus to read from.
+static corpus_sptr
+read_corpus(options			opts,
+	    abigail::fe_iface::status&	status,
+	    const vector<char**>	di_roots,
+	    environment		&env,
+	    const string		&path)
+{
+  corpus_sptr retval = NULL;
+  abigail::tools_utils::file_type type =
+    abigail::tools_utils::guess_file_type(path);
+  abigail::fe_iface_sptr rdr;
+
+  switch (type)
+    {
+    case abigail::tools_utils::FILE_TYPE_UNKNOWN:
+      emit_prefix(opts.prog_name, cerr)
+	<< "Unknown content type for file " << path << "\n";
+      break;
+    case abigail::tools_utils::FILE_TYPE_ELF:
+      {
+#ifdef WITH_CTF
+	if (opts.use_ctf)
+	  {
+	    rdr = ctf::create_reader(path, env);
+	    ABG_ASSERT(rdr);
+
+	    retval = ctf::read_corpus(rdr.get(), status);
+	  }
+	else
+#endif
+	  retval = dwarf::read_corpus_from_elf(path, di_roots, env,
+					       /*load_all_types=*/opts.weak_mode,
+					       status);
+      }
+      break;
+    case abigail::tools_utils::FILE_TYPE_XML_CORPUS:
+      {
+	rdr = abixml::create_reader(path, env);
+	assert(rdr);
+	retval = rdr->read_corpus(status);
+      }
+      break;
+    case abigail::tools_utils::FILE_TYPE_AR:
+    case abigail::tools_utils::FILE_TYPE_XML_CORPUS_GROUP:
+    case abigail::tools_utils::FILE_TYPE_RPM:
+    case abigail::tools_utils::FILE_TYPE_SRPM:
+    case abigail::tools_utils::FILE_TYPE_DEB:
+    case abigail::tools_utils::FILE_TYPE_DIR:
+    case abigail::tools_utils::FILE_TYPE_TAR:
+    case abigail::tools_utils::FILE_TYPE_NATIVE_BI:
+      break;
+    }
+
+  return retval;
+}
+
 int
 main(int argc, char* argv[])
 {
@@ -638,19 +746,25 @@ main(int argc, char* argv[])
       return 0;
     }
 
+  if (opts.weak_mode && !opts.lib2_path.empty())
+    {
+      emit_prefix(argv[0], cout)
+        << "WARNING: The \'--weak-mode\' option is used. The "
+	<< opts.lib2_path << " will be ignored automatically\n";
+    }
+
+  if (opts.redundant_opt_set && opts.no_redundant_opt_set)
+    {
+      emit_prefix(argv[0], cerr)
+        << "ERROR: The \'--redundant\' and '--no-redundant' option are in conflict. "
+	<< "Please select only one option to use.\n";
+      return 1;
+    }
+
   ABG_ASSERT(!opts.app_path.empty());
   if (!abigail::tools_utils::check_file(opts.app_path, cerr, opts.prog_name))
     return abigail::tools_utils::ABIDIFF_ERROR;
 
-  abigail::tools_utils::file_type type =
-    abigail::tools_utils::guess_file_type(opts.app_path);
-  if (type != abigail::tools_utils::FILE_TYPE_ELF)
-    {
-      emit_prefix(argv[0], cerr)
-	<< opts.app_path << " is not an ELF file\n";
-      return abigail::tools_utils::ABIDIFF_ERROR;
-    }
-
   // Create the context of the diff
   diff_context_sptr ctxt = create_diff_context(opts);
 
@@ -659,7 +773,7 @@ main(int argc, char* argv[])
   suppressions_type& supprs = ctxt->suppressions();
   bool files_suppressed = (file_is_suppressed(opts.app_path, supprs)
 			   || file_is_suppressed(opts.lib1_path, supprs)
-			   ||file_is_suppressed(opts.lib2_path, supprs));
+			   || file_is_suppressed(opts.lib2_path, supprs));
 
   if (files_suppressed)
     // We don't have to compare anything because a user
@@ -671,21 +785,34 @@ main(int argc, char* argv[])
   char * app_di_root = opts.app_di_root_path.get();
   vector<char**> app_di_roots;
   app_di_roots.push_back(&app_di_root);
-  status status = abigail::dwarf_reader::STATUS_UNKNOWN;
-  environment_sptr env(new environment);
-  corpus_sptr app_corpus=
-    read_corpus_from_elf(opts.app_path,
-			 app_di_roots, env.get(),
-			 /*load_all_types=*/opts.weak_mode,
-			 status);
+  abigail::fe_iface::status status = abigail::fe_iface::STATUS_UNKNOWN;
+  environment env;
 
-  if (status & abigail::dwarf_reader::STATUS_NO_SYMBOLS_FOUND)
+  corpus_sptr app_corpus = read_corpus(opts, status,
+				       app_di_roots, env,
+				       opts.app_path);
+  if (!app_corpus)
+    {
+      emit_prefix(argv[0], cerr) << opts.app_path
+				 << " is not a supported file\n";
+      return abigail::tools_utils::ABIDIFF_ERROR;
+    }
+
+  if (opts.fail_no_debug_info
+      && (status & abigail::fe_iface::STATUS_ALT_DEBUG_INFO_NOT_FOUND)
+      && (status & abigail::fe_iface::STATUS_DEBUG_INFO_NOT_FOUND))
+    {
+      emit_prefix(argv[0], cerr) << opts.app_path
+				 << " does not have debug symbols\n";
+      return abigail::tools_utils::ABIDIFF_ERROR;
+    }
+  if (status & abigail::fe_iface::STATUS_NO_SYMBOLS_FOUND)
     {
       emit_prefix(argv[0], cerr)
 	<< "could not read symbols from " << opts.app_path << "\n";
       return abigail::tools_utils::ABIDIFF_ERROR;
     }
-  if (!(status & abigail::dwarf_reader::STATUS_OK))
+  if (!(status & abigail::fe_iface::STATUS_OK))
     {
       emit_prefix(argv[0], cerr)
 	<< "could not read file " << opts.app_path << "\n";
@@ -714,29 +841,31 @@ main(int argc, char* argv[])
   ABG_ASSERT(!opts.lib1_path.empty());
   if (!abigail::tools_utils::check_file(opts.lib1_path, cerr, opts.prog_name))
     return abigail::tools_utils::ABIDIFF_ERROR;
-  type = abigail::tools_utils::guess_file_type(opts.lib1_path);
-  if (type != abigail::tools_utils::FILE_TYPE_ELF)
-    {
-      emit_prefix(argv[0], cerr) << opts.lib1_path << " is not an ELF file\n";
-      return abigail::tools_utils::ABIDIFF_ERROR;
-    }
 
   char * lib1_di_root = opts.lib1_di_root_path.get();
   vector<char**> lib1_di_roots;
   lib1_di_roots.push_back(&lib1_di_root);
-  corpus_sptr lib1_corpus = read_corpus_from_elf(opts.lib1_path,
-						 lib1_di_roots, env.get(),
-						 /*load_all_types=*/false,
-						 status);
-  if (status & abigail::dwarf_reader::STATUS_DEBUG_INFO_NOT_FOUND)
-    emit_prefix(argv[0], cerr)
-      << "could not read debug info for " << opts.lib1_path << "\n";
-  if (status & abigail::dwarf_reader::STATUS_NO_SYMBOLS_FOUND)
+  corpus_sptr lib1_corpus = read_corpus(opts, status,
+					lib1_di_roots,
+					env, opts.lib1_path);
+  if (!lib1_corpus)
     {
-      cerr << "could not read symbols from " << opts.lib1_path << "\n";
+      emit_prefix(argv[0], cerr) << opts.lib1_path
+				 << " is not a supported file\n";
       return abigail::tools_utils::ABIDIFF_ERROR;
     }
-  if (!(status & abigail::dwarf_reader::STATUS_OK))
+  if (opts.fail_no_debug_info
+      && (status & abigail::fe_iface::STATUS_ALT_DEBUG_INFO_NOT_FOUND)
+      && (status & abigail::fe_iface::STATUS_DEBUG_INFO_NOT_FOUND))
+    emit_prefix(argv[0], cerr)
+      << "could not read debug info for " << opts.lib1_path << "\n";
+  if (status & abigail::fe_iface::STATUS_NO_SYMBOLS_FOUND)
+    {
+      emit_prefix(argv[0], cerr) << "could not read symbols from "
+				 << opts.lib1_path << "\n";
+      return abigail::tools_utils::ABIDIFF_ERROR;
+    }
+  if (!(status & abigail::fe_iface::STATUS_OK))
     {
       emit_prefix(argv[0], cerr)
 	<< "could not read file " << opts.lib1_path << "\n";
@@ -751,20 +880,31 @@ main(int argc, char* argv[])
       char * lib2_di_root = opts.lib2_di_root_path.get();
       vector<char**> lib2_di_roots;
       lib2_di_roots.push_back(&lib2_di_root);
-      lib2_corpus = read_corpus_from_elf(opts.lib2_path,
-					 lib2_di_roots, env.get(),
-					 /*load_all_types=*/false,
-					 status);
-      if (status & abigail::dwarf_reader::STATUS_DEBUG_INFO_NOT_FOUND)
-	emit_prefix(argv[0], cerr)
-	  << "could not read debug info for " << opts.lib2_path << "\n";
-      if (status & abigail::dwarf_reader::STATUS_NO_SYMBOLS_FOUND)
+      lib2_corpus = read_corpus(opts, status,
+				lib2_di_roots, env,
+				opts.lib2_path);
+      if (!lib2_corpus)
+	{
+	  emit_prefix(argv[0], cerr) << opts.lib2_path
+				     << " is not a supported file\n";
+	  return abigail::tools_utils::ABIDIFF_ERROR;
+	}
+
+      if (opts.fail_no_debug_info
+	  && (status & abigail::fe_iface::STATUS_ALT_DEBUG_INFO_NOT_FOUND)
+	  && (status & abigail::fe_iface::STATUS_DEBUG_INFO_NOT_FOUND))
+	{
+	  emit_prefix(argv[0], cerr)
+	    << "could not read debug info for " << opts.lib2_path << "\n";
+	  return abigail::tools_utils::ABIDIFF_ERROR;
+	}
+      if (status & abigail::fe_iface::STATUS_NO_SYMBOLS_FOUND)
 	{
 	  emit_prefix(argv[0], cerr)
 	    << "could not read symbols from " << opts.lib2_path << "\n";
 	  return abigail::tools_utils::ABIDIFF_ERROR;
 	}
-      if (!(status & abigail::dwarf_reader::STATUS_OK))
+      if (!(status & abigail::fe_iface::STATUS_OK))
 	{
 	  emit_prefix(argv[0], cerr)
 	    << "could not read file " << opts.lib2_path << "\n";
